#!@PERL@
#
# Main -- an exercise in initrd building
#   Copyright (C) 2005  Erik van Konijnenburg
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#
use strict;
use warnings;
use lib '@PKGDATADIR@';

use Getopt::Long;
use Plan;
use TestSet;
use Pack;

my $appName = "@PACKAGE@";
my $appVersion = "@VERSION@";


#
# go -- main program: given name of directory, writes proposed
# contents for initrd to it. directory must not yet exist.
#
sub go ($$) {
	my ($format, $destination) = @_;
	my $masterPlan = Plan::makePlan ();
	my $image = $masterPlan->expand ();
	Pack::package ($image, $format, $destination);
}


#
# paranoia -- Random bits of -
#
sub paranoia {
	umask 077;

	#
	# Perlsec(1) identifies the following as risky:
	#
	#	delete @ENV{qw(IFS CDPATH ENV BASH_ENV)}; 
	#
	# but just scrapping the complete environment
	# seems to work nicely.
	#
	delete @ENV{keys %ENV};
	$ENV{'PATH'} = '/bin:/sbin:/usr/bin:/usr/sbin';
}


#
# usage -- print invocation patterns of the application
#
sub usage {
	print <<"//";

$appName generates initrd images.

$appName --test [ version ]
	print an overview of system data, without writing anything to disk.

$appName --output dest --format format [ version ]
	generate an initrd image in 'dest', in the format 'format'.

Options:
   -h, -?, --help	print this help text and exit
   -V, --version	print version and exit (don't bother: $appVersion)
   -v, --verbose	print progress messages
   -d, --debug		detailed output
   -q, --quiet		don't print warning messages
   -o, --output dest	place result here
   -f, --format form	produce the image in this format,
			valid values: directory, cramfs, cpio
   -t, --test		print overview of system data,
			used as basis for debugging.

//
}


#
# main -- main program, argument processing
#
sub main {
	my ($opt_help, $opt_version,
		$opt_verbose, $opt_quiet, $opt_debug,
		$opt_format, $opt_output, $opt_test);

	paranoia ();
	Base::setProgName ($appName);

	my @warnings = ();
	{
		#
		# catch warnings during getopt, but not elsewhere.
		# The idea is to let -? produce help without complaint,
		# even if other args are broken.
		#
		local $SIG{__WARN__} = sub { push @warnings, $_[0]; };

		Getopt::Long::Configure ("bundling");
		my $result = GetOptions (
			"h|help|?"	=> \$opt_help,
			"V|version"	=> \$opt_version,
			"v|verbose"	=> \$opt_verbose,
			"q|quiet"	=> \$opt_quiet,
			"d|debug"	=> \$opt_debug,
			"f|format=s"	=> \$opt_format,
			"o|output=s"	=> \$opt_output,
			"t|test"	=> \$opt_test,
			);
		}

	#
	# --help and --version processed without regard to other options,
	# exit without side effect.
	#
	if (defined ($opt_help)) {
		usage ();
		Base::bye();
	}
	if (defined ($opt_version)) {
		Base::setVerbose (1);
		Base::info ("version $appVersion");
		Base::bye();
	}

	#
	# Argument validation
	#
	for my $w (@warnings) {
		# lazy: perhaps we could provide more info here?
		Base::fatal ("$w");
	}
	if (defined ($opt_output) && defined ($opt_test)) {
		Base::fatal "conflicting options --output and --test";
	}
	if (! defined ($opt_output) && ! defined ($opt_test)) {
		Base::fatal "one of --output and --test must be given";
	}
	if (defined ($opt_output)) {
		if (-e $opt_output) {
			Base::fatal "destination $opt_output already exists";
		}
		if (! defined ($opt_format)) {
			Base::fatal "option --output requires --format option";
		}
		if (! Pack::knownFormat ($opt_format)) {
			Base::fatal "unknown --format argument: $opt_format";
		}

		#
		# Untaint -- is there a need for 'output' with an umlaut?
		#
		if ($opt_output !~ m!^([-_./[a-zA-Z0-9]+)$!) {
			Base::fatal ("illegal character in output file name");
		}
		$opt_output = $1;
	}
	if (defined ($opt_test) && defined ($opt_format)) {
		Base::fatal "conflicting options --test and --format";
	}
	if ($#ARGV > 0) {
		my $valid = $ARGV[0];
		Base::fatal "extra arguments after $valid";
	}

	#
	# All arguments validated; start processing.
	#
	if ($#ARGV == 0) {
		Conf::set ("version", $ARGV[0]);
	}
	if ($opt_verbose) {
		Base::setVerbose (1);
	}
	if ($opt_debug) {
		Base::setDebug (1);
	}
	if ($opt_quiet) {
		Base::setQuiet (1);
	}

	# avoid silly messages with an early test
	my $v = Conf::get("version");
	my $mdir = Conf::get("libModules") . "/$v";
	if (! -d $mdir) {
		Base::fatal ("unknown kernel version: $v");
	}

	# go for it.
	if (defined ($opt_test)) {
		TestSet::testAll();
	}
	elsif (defined ($opt_output)) {
		go ($opt_format, $opt_output);
	}
	Base::bye;
}

main ();
